#include "CFicheroMBTiles.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

int CFicheroMBTiles::obtMinZoom(void)
{
 sqlite3_stmt *lpStmt;
 int liErrorCode;
 const unsigned char *lpchrDat;
 int liZoom;


 liErrorCode=sqlite3_prepare_v2(mpConecBD, "SELECT value FROM metadata WHERE name=\"minzoom\";", -1, &lpStmt, 0);
 if (liErrorCode!= SQLITE_OK)
  return -1;

 liZoom=0;
 liErrorCode=sqlite3_step(lpStmt);
 while (liErrorCode==SQLITE_ROW)
 {
  lpchrDat=sqlite3_column_text(lpStmt,0);
  liZoom=atoi((const char *)lpchrDat);
  liErrorCode=sqlite3_step(lpStmt);
 }
 sqlite3_finalize(lpStmt);
 return liZoom;
}


const char *CFicheroMBTiles::obtFormat(void)
{
 sqlite3_stmt *lpStmt;
 int liErrorCode;
 const char *lpchrDat;

 lpchrDat=NULL;
 liErrorCode=sqlite3_prepare_v2(mpConecBD, "SELECT value FROM metadata WHERE name=\"format\";", -1, &lpStmt, 0);
 if (liErrorCode!= SQLITE_OK)
  return NULL;

 liErrorCode=sqlite3_step(lpStmt);
 while (liErrorCode==SQLITE_ROW)
 {
  lpchrDat=_strdup((const char *)sqlite3_column_text(lpStmt,0));
  liErrorCode=sqlite3_step(lpStmt);
 }

 sqlite3_finalize(lpStmt);
 return lpchrDat;
}

/*Construye la SQL para obtener los datos de una tile
 * Entrada:
 * 	Zoom a obtener informacion
 * 	Que se selecciona:
 * 		"c": devuelve los campos zoom_level,tile_column,tile_row (SELECT zoom_level,tile_column,tile_row)
 * 		"g": devuelve la informacion gráfica tile_data (SELECT tile_data)
 * 	        "*": devuelve todos los campos de la tabla tiles (SELECT *)
 * 	       "dz": devuelve los zooms zoom_level (SELECT DISTINCT zoom_level)
 * 	criterio seleccion
 * 		'=' igual
 * 		'>' mayor igual
 * Salida:
 * 	SQL
 *
 */
unsigned char *CFicheroMBTiles::buildSQLTiles(int piZoom, char pchSelect[2],char pchCriterio[2])
{
 unsigned char *lpchrSQL;
 char lchrWhere[50];


 lpchrSQL=new unsigned char[200];
 lpchrSQL[0]='\0';


 if (strcoll("c",pchSelect)==0)
  strcat((char *)lpchrSQL,"SELECT zoom_level,tile_column,tile_row\n");
  //_mbsnbcat_s(lpchrSQL,200,(unsigned char *)"SELECT zoom_level,tile_column,tile_row\n",39);
 else
    if (strcoll("g",pchSelect)==0)
     strcat((char *)lpchrSQL,"SELECT tile_data\n");
     //_mbsnbcat_s(lpchrSQL,200,(unsigned char *)"SELECT tile_data\n",17);
    else
       if (strcoll("*",pchSelect)==0)
	strcat((char *)lpchrSQL,"SELECT zoom_level,tile_row,tile_column,tile_data\n");
	//_mbsnbcat_s(lpchrSQL,200,(unsigned char *)"SELECT zoom_level,tile_row,tile_column,tile_data\n",49);
       else
	  if (strcoll("dz",pchSelect)==0)
	   strcat((char *)lpchrSQL,"SELECT distinct zoom_level\n");
	   //_mbsnbcat_s(lpchrSQL,200,(unsigned char *)"SELECT distinct zoom_level\n",27);
	  else
             {
	      delete[] lpchrSQL;
	      lpchrSQL=NULL;
	      return lpchrSQL;
             }

 strcat((char *)lpchrSQL,"FROM tiles\n");
 //_mbsnbcat_s(lpchrSQL,200,(unsigned char *)"FROM tiles\n",11);
 if (strcoll(">=",pchCriterio)==0)
  snprintf(lchrWhere,50,"WHERE zoom_level>=%i\n",piZoom);
 else
    if (strcoll("=",pchCriterio)==0)
     snprintf(lchrWhere,50,"WHERE zoom_level=%i\n",piZoom);
 //_mbsnbcat_s(lpchrSQL,200,(unsigned char *)lchrWhere,50);
 //_mbsnbcat_s(lpchrSQL,200,(unsigned char *)"ORDER BY zoom_level,tile_row desc,tile_column;\n",47);
 strcat((char *)lpchrSQL,lchrWhere);
 strcat((char *)lpchrSQL,"ORDER BY zoom_level,tile_row desc,tile_column;\n");

 return lpchrSQL;
}

tile CFicheroMBTiles::crearTile(int piZoom, int piCol, int piRow, int piSizeDatGrafico, const void *pDatGrafico)
{
 tile lTile;

 lTile.zoom=piZoom;
 lTile.columna=piCol;
 lTile.fila=piRow;
 lTile.sizeBytes=piSizeDatGrafico;
 //lTile.pVoid=pDatGrafico;
 lTile.pRawImg=new unsigned char [piSizeDatGrafico];
 //memcpy_s(lTile.pRawImg,piSizeDatGrafico,pDatGrafico,piSizeDatGrafico);
 memcpy(lTile.pRawImg,pDatGrafico,piSizeDatGrafico);

 return lTile;
}

unsigned char CFicheroMBTiles::obtZooms(int piZoom)
{
 unsigned char *lpchrSQL;
 sqlite3_stmt *lpStmt;
 int liErrorCode;
 int liZoom;
 char lchrOperacion[3]=">=";
 char lchrSelect[]="dz";

 mVecZooms.clear();

 lpchrSQL=buildSQLTiles(piZoom,lchrSelect,lchrOperacion);

 liErrorCode=sqlite3_prepare_v2(mpConecBD, (const char *)lpchrSQL, -1, &lpStmt, 0);
 if (liErrorCode!= SQLITE_OK)
  return -1;

 liErrorCode=sqlite3_step(lpStmt);
 while (liErrorCode==SQLITE_ROW)
 {
  liZoom=sqlite3_column_int(lpStmt,0);
  mVecZooms.push_back(liZoom);
  liErrorCode=sqlite3_step(lpStmt);
 }//fin while

 sqlite3_finalize(lpStmt);
 delete lpchrSQL;
 return 0;
}

/*
 * Borrar todas las tiles (vacia el vector mVecTiles)
 */
void CFicheroMBTiles::borrarTiles(void)
{
 std::vector<tile>::iterator lIteratorTile;
 tile lInfoTile;

 if (mVecTiles.size()<=0)
  return;

 for (lIteratorTile=mVecTiles.begin();lIteratorTile!=mVecTiles.end(); lIteratorTile++)
 {
  lInfoTile=*lIteratorTile;
  if (lInfoTile.pRawImg!=NULL)
  {
   delete []lInfoTile.pRawImg;
   lInfoTile.pRawImg=NULL;
  }
 }
 mVecTiles.clear();
}


CFicheroMBTiles::CFicheroMBTiles ()
{
 mpConecBD=NULL;
 mfLeft=0.0f;
 mfBottom=0.0f;
 mfRight=0.0f;
 mfTop=0.0f;
 mpchFormato=NULL;
 mpstrPathFile=NULL;
 miMinColZoom=0;
 miMaxFilaZoom=0;
}

CFicheroMBTiles::~CFicheroMBTiles ()
{
 if (mpchFormato!=NULL)
  delete mpchFormato;
 if (mpstrPathFile!=NULL)
  delete mpstrPathFile;

 borrarTiles();
}

/* Carraga los datos (no las imagnes) del fichero MBTiles.
 * Entrada:
 * 	pstrPathFile:Ruta y fichero a cargar
 * Salida:
 * 	Estado:
 * 	  0: Correcto
 * 	-10: Error a ralizar la conecsion conla BD.
 * 	-20: Error al obtener el parámetro zoom
 * 	-30: Error al obtener el parametro formato
 * 	-40: Error no es un formato PNG
 * 	-50: Error al obtener la informacion de los zooms
 */
char CFicheroMBTiles::leerFichero(const char *ppstrPathFile)
{
 int liErrorCode;
 int liZoomMin;


 liErrorCode=sqlite3_open_v2(ppstrPathFile, &mpConecBD, SQLITE_OPEN_READONLY, NULL);
 if (liErrorCode!=SQLITE_OK)
 {
  sqlite3_close_v2(mpConecBD);
  mpConecBD=NULL;
  //error al realizar la conexion
  return -10;
 }

 mpstrPathFile=_strdup(ppstrPathFile);
 //leo los datos del metadata
 //obtener minzoo
 liZoomMin=obtMinZoom();
 if (liZoomMin<0)
 {
  sqlite3_close(mpConecBD);
  mpConecBD=NULL;
  //no tiene el parametro minzoom
  return -20;
 }
 //obtengo el formato del grafico
 mpchFormato=(const unsigned char *)obtFormat();
 if (mpchFormato==NULL)
 {
  sqlite3_close(mpConecBD);
  mpConecBD=NULL;
  //no tiene el parametro format
  return -30;
 }

 //comparamos ignorando mayusculas, minusculas
 /*if (_strnicmp((const char *)"PNG",(const char *)mpchFormato,3)!=0)
 {
  if (_strnicmp((const char *)"JPG",(const char *)mpchFormato,3)!=0)
  {
   sqlite3_close(mpConecBD);
   mpConecBD=NULL;
   //formato desconocido
   return -40;
  }
 }*/

 if (strcmp((const char *)"PNG",(const char *)mpchFormato)!=0)
 {
  if (strcmp((const char *)"png",(const char *)mpchFormato)!=0)
  {
   if (strcmp((const char *)"JPG",(const char *)mpchFormato)!=0)
   {
    if (strcmp((const char *)"jpg",(const char *)mpchFormato)!=0)
    {
     sqlite3_close(mpConecBD);
     mpConecBD=NULL;
     //formato desconocido
     return -40;
    }
   }
  }
 }

 //obtenemos los zooms
 if (obtZooms(liZoomMin)<0)
 {
  sqlite3_close(mpConecBD);
  mpConecBD=NULL;
  //error al obtener los zooms
  return -50;
 }


 sqlite3_close(mpConecBD);
 mpConecBD=NULL;
 return 0;
}


/*Devuelve el formato de los tiles
 * Entrada:
 * Salida:
 * 	formato de los tiles
 */
const unsigned char *CFicheroMBTiles::obtFormatoImg(void)
{
 return mpchFormato;
}


/*Devuelve el numero de zooms
 * Entrada:
 * Salida:
 * 	Numero de zooms
 */
unsigned int CFicheroMBTiles::obtNumZooms(void)
{
 return (unsigned int)mVecZooms.size();
}

/*Devuleve el zoom que hay en el indice (posicion del vector, el vector empiza en 0)
 * Entrada:
 * 	Indice del zoom
 * Salida:
 * 	Zoom que hay en dicho indice
 *
 */
int CFicheroMBTiles::obtInfoPosZoom(unsigned int puiIndi)
{
 size_t lsztyLong;


 lsztyLong=mVecZooms.size();

 if (lsztyLong<=0)
 {
  return -1;
 }

 if (puiIndi>lsztyLong)
 {
  return 2;
 }

 return mVecZooms[puiIndi];
}


/*Busca un determinado zoom
 * Entrada:
 * 	Zoom a buscar
 * Salida:
 * 	Indice del vector encontrado
 * 	-1 No hay zooms
 * 	-2 Zoom no encontrado
 *
 */
int CFicheroMBTiles::findZoom(int piZoom)
{
 size_t lsztyLong;
 std::vector<int>::iterator lIteratorZoom;
 int liZoom;
 int liIndi;
 bool lbEnc;


 lsztyLong=mVecZooms.size();

 if (lsztyLong<=0)
 {
  return -1;
 }

 liIndi=0;
 lbEnc=false;
 for (lIteratorZoom=mVecZooms.begin();lIteratorZoom!=mVecZooms.end(); lIteratorZoom++)
 {
  liZoom=*lIteratorZoom;
  if (piZoom==liZoom)
  {
   lbEnc=true;
   break;
  }
  liIndi++;
 }

 if (lbEnc==false)
  return -2;

 return liIndi;
}

/*Cargamos los tiles para un determinado zoom
 * Entrada:
 * 	Zoom a cargar
 * Salida:
 * 	Estado:
 * 	  0: Correcto
 * 	 -1: No existe ruta y nombre fichero
 * 	 -2: No se ha podido construir la SQL
 * 	-10: Error en la conexion con la BD
 * 	-20: Error al ejecutar la SQL
 *
 */
char CFicheroMBTiles::cargarTiles(int piZoom, unsigned int *ppuiNumTiles)
{
 unsigned char *lpchrSQL;
 sqlite3_stmt *lpStmt;
 int liErrorCode;
 int liZoom;
 int liColumn;
 int liRow;
 //tamaño en bytes del dato tile_data
 int liTamaTile;
 char lchrOperacion[2]="=";
 char lchrSelect[]="*";

 if (mpstrPathFile==NULL)
  return -1;

 //indicamos que en la sql aparecan todos los campos
 lpchrSQL=buildSQLTiles(piZoom,lchrSelect,lchrOperacion);
 if (lpchrSQL==NULL)
  return -2;


 //abrimos conexion con la base de datos
 liErrorCode=sqlite3_open_v2(mpstrPathFile, &mpConecBD, SQLITE_OPEN_READONLY, NULL);
 if (liErrorCode!=SQLITE_OK)
 {
  sqlite3_close_v2(mpConecBD);
  mpConecBD=NULL;
  //error al realizar la conexion
  delete []lpchrSQL;
  return -10;
 }

 //ejecutamos la sql

 liErrorCode=sqlite3_prepare_v2(mpConecBD, (const char *)lpchrSQL, -1, &lpStmt, 0);
 if (liErrorCode!= SQLITE_OK)
 {
  sqlite3_close_v2(mpConecBD);
  mpConecBD=NULL;
  delete []lpchrSQL;
  return -20;
 }

 //limpiamos el vector de los tiles
 mVecTiles.clear();
 miMinColZoom=-1;
 miMaxFilaZoom=0;
 liErrorCode=sqlite3_step(lpStmt);
 while (liErrorCode==SQLITE_ROW)
 {//cursor
  liZoom=sqlite3_column_int(lpStmt,0);
  liRow=sqlite3_column_int(lpStmt,1);
  liColumn=sqlite3_column_int(lpStmt,2);
  liTamaTile=sqlite3_column_bytes(lpStmt,3);
  mVecTiles.push_back(crearTile(liZoom,liColumn,liRow,liTamaTile,sqlite3_column_blob(lpStmt, 3)));
  if (miMaxFilaZoom<liRow)
   miMaxFilaZoom=liRow;

  if (miMinColZoom==-1)
   miMinColZoom=liColumn;
  else
     if (miMinColZoom>liColumn)
      miMinColZoom=liColumn;
  liErrorCode=sqlite3_step(lpStmt);
 }//fin cursor

 *ppuiNumTiles=(unsigned int)mVecTiles.size();

 sqlite3_finalize(lpStmt);

 sqlite3_close_v2(mpConecBD);
 mpConecBD=NULL;
 delete []lpchrSQL;
 return 0;
}

//Devuleve el número de tiles cargados despues de ejcutar la funcion cargarTiles
unsigned int CFicheroMBTiles::obtNumTilesCargados(void)
{
 return (unsigned int)mVecTiles.size();
}


/*Devuleve la informacion de un determinado tile
 * Entrada:
 * 	Indice del zoom
 * Salida:
 * 	tile
 * 	Estado de la operacion
 *
 */
tile CFicheroMBTiles::obtTileByIndice(unsigned int puiIndi, char *ppchEstado)
{
 tile linfoTileResult,linfoTileDat;
 size_t lsztyLong;

 lsztyLong=mVecTiles.size();

 if (lsztyLong<=0)
 {
  *ppchEstado=-1;
  return linfoTileResult;
 }

 if (puiIndi>=lsztyLong)
 {
  *ppchEstado=-2;
  return linfoTileResult;
 }

 linfoTileDat=mVecTiles[puiIndi];
 linfoTileResult.zoom=linfoTileDat.zoom;
 linfoTileResult.fila=linfoTileDat.fila;
 linfoTileResult.columna=linfoTileDat.columna;
 linfoTileResult.sizeBytes=linfoTileDat.sizeBytes;
 linfoTileResult.pRawImg=linfoTileDat.pRawImg;
 *ppchEstado=0;
 return linfoTileResult;
}

/*Sólo borra los dataos de la imagen(rawImg), el zoom, y posicion (Columna,Fila)
 * no se borrar
 * Entrada:
 * 	Indice del zoom
 * Salida:
 * 	Estado:
 * 	  0:Correcto
 * 	 -1:No hay elementos a borrar
 * 	 -2:Indice fuera de rango
 */
char CFicheroMBTiles::borrarRawImg(unsigned int puiIndi)
{
 size_t lsztyLong;

 lsztyLong=mVecTiles.size();
 if (lsztyLong<=0)
 {
  return -1;
 }

 if (puiIndi>=lsztyLong)
 {
  return -2;
 }

 delete []mVecTiles[puiIndi].pRawImg;
 mVecTiles[puiIndi].pRawImg=NULL;
 mVecTiles[puiIndi].sizeBytes=0;

 return 0;
}


/*Obtenemos la columna minima del zoom cargado
 * Entrada:
 * 	Zoom
 * Salida:
 * 	Minimo zoom del zoom, <0 error
 */
int CFicheroMBTiles::obtMinColumna(void)
{
 return miMinColZoom;
}


/*Obtenemos la fila maxima del zoom cargado
 * Salida:
 * 	Minimo zoom del zoom, <0 error
 */
int CFicheroMBTiles::obtMaxFila(void)
{
 return miMaxFilaZoom;
}

