/*
 * Para quitar la dependencia de wxWigdets, en lugar de guardar las imagenes en wxBitmap
 * las guradmos en un puntero a unsigned char.
 */

#ifndef CFICHEROMBTILES_CFICHEROMBTILES_H_
#define CFICHEROMBTILES_CFICHEROMBTILES_H_


#include <sqlite3.h>
#include <vector>


//define la unidad minima de informacion los tiles
typedef struct
{
 int zoom;
 int columna;
 int fila;
 int sizeBytes;
 unsigned char *pRawImg;
} tile;


class CFicheroMBTiles
{
 private:
  sqlite3 *mpConecBD;
  const char *mpstrPathFile;
  float mfLeft;
  float mfBottom;
  float mfRight;
  float mfTop;
  const unsigned char *mpchFormato;
  int miMinColZoom;
  int miMaxFilaZoom;

  //guardamos los zooms que compone el mapa
  std::vector<int>mVecZooms;

  //para un zoom dado guardamos sus tiles
  std::vector<tile> mVecTiles;

  //obtenemos el zoom mínimo
  int obtMinZoom(void);
  //obtenemos el formato de la imagen
  const char *obtFormat(void);
  //obtenemos los zooms
  unsigned char obtZooms(int piZoom);

  /*Construye la SQL para obtener los datos de una tile
   * Entrada:
   * 	Zoom a obtener informacion
   * 	Que se selecciona:
   * 		"c": devuelve los campos zoom_level,tile_column,tile_row (SELECT zoom_level,tile_column,tile_row)
   * 		"g": devuelve la informacion gráfica tile_data (SELECT tile_data)
   * 	        "*": devuelve todos los campos de la tabla tiles (SELECT *)
   * 	       "dz": devuelve los zooms zoom_level (SELECT DISTINCT zoom_level)
   * 	criterio seleccion
   * 		'=' igual
   * 		'>' mayor igual
   * Salida:
   * 	SQL
   *
   */
  unsigned char *buildSQLTiles(int piZoom,char pchSelect[2], char pchCriterio[2]);

  tile crearTile(int piZoom, int piCol, int piRow, int piSizeDatGrafico, const void *pDatGrafico);



 public:
  CFicheroMBTiles ();
  virtual ~CFicheroMBTiles ();

  /*Carraga los datos (no las imagnes) del fichero MBTiles.
   * Entrada:
   * 	pstrPathFile:Ruta y fichero a cargar
   * Salida:
   * 	Estado:
   * 	  0: Correcto
   * 	-10: Error a ralizar la conecsion conla BD.
   * 	-20: Error al obtener el parámetro zoom
   * 	-30: Error al obtener el parametro formato
   * 	-40: Error no es un formato PNG
   * 	-50: Error al obtener la informacion de los zooms
   */
  char leerFichero(const char *ppstrPathFile);

  /*Devuelve el formato de los tiles
   * Entrada:
   * Salida:
   * 	formato de los tiles
   */
  const unsigned char *obtFormatoImg(void);

  /*Devuelve el numero de zooms
   * Entrada:
   * Salida:
   * 	Numero de zooms
   */
  unsigned int obtNumZooms(void);

  /*Devuleve el zoom que hay en el indice (posicion del vector, el vector empiza en 0)
   * Entrada:
   * 	Indice del zoom
   * Salida:
   * 	Zoom que hay en dicho indice
   *
   */
  int obtInfoPosZoom(unsigned int puiIndi);

  /*Busca un determinado zoom
   * Entrada:
   * 	Zoom a buscar
   * Salida:
   * 	Indice del vector encontrado
   * 	-1 No hay zooms
   * 	-2 Zoom no encontrado
   *
   */
  int findZoom(int piZoom);

  /*Cargamos los tiles para un determinado zoom
   * Entrada:
   * 	Zoom a cargar
   * Salida:
   * 	Estado:
   * 	  0: Correcto
   * 	 -1: No existe ruta y nombre fichero
   * 	 -2: No se ha podido construir la SQL
   * 	-10: Error en la conexion con la BD
   * 	-20: Error al ejecutar la SQL
   *
   */
  char cargarTiles(int piZoom, unsigned int *ppuiNumTiles);

  /*Devuleve el número de tiles cargados despues de ejecutar la función cargarTiles
   */
  unsigned int obtNumTilesCargados(void);

  /*Devuleve la información de un determinado tile
   * Entrada:
   * 	Indice del zoom
   * Salida:
   * 	tile
   * 	Estado de la operacion
   *
   */
  tile obtTileByIndice(unsigned int puiIndi ,char *ppchEstado);

  /*Sólo borra los dataos de la imagen(rawImg), el zoom, y posición (Columna,Fila)
   * no se borrar
   * Entrada:
   * 	Indice del zoom
   * Salida:
   * 	Estado:
   * 	  0:Correcto
   * 	 -1:No hay elementos a borrar
   * 	 -2:Indice fuera de rango
   */
   char borrarRawImg(unsigned int puiIndi);

   /*
    * Borrar todas los tiles (vacia el vector mVecTiles)
    */
   void borrarTiles(void);

   /*Obtenemos la columna mínima del zoom cargado
    * Entrada:
    * 	Zoom
    * Salida:
    * 	Minimo zoom del zoom, <0 error
    */
   int obtMinColumna(void);

   /*Obtenemos la fila máxima del zoom cargado
    * Salida:
    * 	Minimo zoom del zoom, <0 error
    */
   int obtMaxFila(void);
};

#endif /* CFICHEROMBTILES_CFICHEROMBTILES_H_ */
